function varargout = bluenile(varargin)
% BLUENILE filters images in the frequency domain
%
% ---
%
% Vlad Atanasiu * 02/09/2003
% Last revision > 25/11/2006
%
% atanasiu@excite.com
% Language & Calligraphy:
% http://mywebpage.netscape.com/atanasiuvlad/frq/

% BLUENILE M-file for bluenile.fig
%      BLUENILE, by itself, creates a new BLUENILE or raises the existing
%      singleton*.
%
%      H = BLUENILE returns the handle to a new BLUENILE or the handle to
%      the existing singleton*.
%
%      BLUENILE('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in BLUENILE.M with the given input arguments.
%
%      BLUENILE('Property','Value',...) creates a new BLUENILE or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before bluenile_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to bluenile_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @bluenile_OpeningFcn, ...
                   'gui_OutputFcn',  @bluenile_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin & ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --------------------------------------------------------------------
% Executes just before bluenile is made visible.
% --------------------------------------------------------------------
function bluenile_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to GUI_figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to bluenile (see VARARGIN)

% Choose default command line output for bluenile
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes bluenile wait for user response (see UIRESUME)
% uiwait(handles.figure1);
axes(handles.in_axes), axis off
axes(handles.fft_axes), axis off
axes(handles.out_axes), axis off
axes(handles.splash_axes)
imshow('bluenile/splash.jpg')
title('')
axis off

% --------------------------------------------------------------------
% Outputs from this function are returned to the command line.
% --------------------------------------------------------------------
function varargout = bluenile_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to GUI_figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --------------------------------------------------------------------
% LOAD IMAGE
% --------------------------------------------------------------------
function Load_pushbutton_Callback(hObject, eventdata, handles)
% hObject    handle to Load_pushbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global pathname filename imgNow numberOfImages ...
    ImgIn ImgInHue ImgInSat colorImg inImgHandle
global R R2 Z I2 I2C M isotop

% initialisation
imgNow = 1;
numberOfImages = 1;
skippedFiles = 0;

% repeat analysis for all images
while imgNow <= numberOfImages

    if imgNow == 1
        % select image of allowed file formats
        [filename,pathname] = uigetfile(...
            {'*.jpg;*.jpeg;*.tif;*.tiff;*.bmp;*.png;*.pcx;*.hdf;*.xwd', '> Images'; ...
                '*.jpg;*.jpeg', 'JPG, JPEG';...
                '*.tif;*.tiff', 'TIFF, TIFF'; ...
                '*.bmp',        'BMP'; ...
                '*.png',        'PNG'; ...
                '*.pcx',        'PCX'; ...
                '*.hdf',        'HDF'; ...
                '*.xwd',        'XWD'; ...
                '*.*',          'All Files (*.*)'}, ...
            'Load image');
        
        % exit if no file or no directory selected
        if filename ~= 0 | pathname ~= 0
            % continue
        else
            return
        end
        
        % change current directory
        cd(pathname)
        
        % make list of images to batch
        if get(handles.batch_checkbox,'Value') == 1
            InNameBatch = dir;
            InNameBatch = InNameBatch(3:size(InNameBatch));
            
            % get number of images to analyse
            numberOfImages = size(InNameBatch,1);
        else
            InFileFullName = [pathname,filename];
        end
    end
    
    if get(handles.batch_checkbox,'Value') == 1
        % sets image name depending on number of images to analyse
        filename = char(InNameBatch(imgNow).name);
    end
    
    % loads input image if any
    try
        ImgIn = imread([pathname,filename]);
        imgNow = imgNow + 1;
    catch
        % error handling    
        if get(handles.batch_checkbox,'Value') == 0
            % stop if can't load first image   
            errordlg(lasterr,'File Type Error','modal');
            return
        else
            % skip file that cannot be loaded
            imgNow = imgNow + 1;
            continue
        end
    end
    
    colorImg = 0;
    % filtering of RGB images is performed on only the value part of the HSV
    if isrgb(ImgIn) == 1
        colorImg = 1;
        ImgIn = rgb2hsv(ImgIn);
        ImgInHue = ImgIn(:,:,1);
        ImgInSat = ImgIn(:,:,2);
        ImgIn = ImgIn(:,:,3);
    end
    
    % set toolbox preferences
    iptsetpref('ImshowAxesVisible','on')
    set(gcf,'Name',['BlueNile - ',...
            pathname,filename,...
            ' - Image ',num2str(imgNow-1),' of ',num2str(numberOfImages)])
    
    % clear axes
    if ishandle(handles.splash_axes)
        delete(handles.splash_axes);
    end
    axes(handles.in_axes), cla
    axes(handles.fft_axes), cla
    axes(handles.out_axes), cla

    % announce activity status
    set(handles.filter_text,'BackgroundColor','r')
    
    % display image in input axes
    axes(handles.in_axes)
    if colorImg == 1;
        inImgHandle = imshow(hsv2rgb(cat(3,ImgInHue,ImgInSat,ImgIn)));
    else
        inImgHandle = imshow(ImgIn);
    end
    set(handles.in_axes,'TickDir','out')
    title('Input image');
    coloring_checkbox_Callback(hObject, [], handles);
    drawnow
    
    % autocreate lowpass filter
    if get(handles.auto_checkbox,'Value') == 1
        auto_checkbox_Callback(hObject, eventdata, handles);
    end
    
    % display FFT & inverse transform
    DoFft(hObject,handles);
    % no filtering yet
    R2 = R;
    % initialize filter mask
    M = ones(size(ImgIn,1),size(ImgIn,2));
    DoInvFft(hObject,eventdata,handles);
    
    % filter FFT
    DoFilter(hObject,eventdata,handles);
    
    % reset toolbox preferences
    iptsetpref('ImshowAxesVisible','off')
    guidata(hObject, handles);
    
    % save images if batching
    if get(handles.batch_checkbox,'Value') == 1
        Save_pushbutton_Callback(hObject,eventdata,handles);
    end

end % end of batch

% --------------------------------------------------------------------
% FFT
% --------------------------------------------------------------------
function DoFft(hObject,handles)
global ImgIn R R2 Z I2 I2C wichFFThandle XYLim

% get magnitude & phase of FFT
F = fftshift(fft2(ImgIn));
R = abs(F);    % magnitude
Z = angle(F);  % phase
% use C = magnitude.*exp(i*phase) to recombine

axes(handles.fft_axes)
axis manual

% put coords of center of image
XYLim = [-floor(size(ImgIn,2)/2),ceil(size(ImgIn,2)/2)-1;...
        -floor(size(ImgIn,1)/2),ceil(size(ImgIn,1)/2)-1];
wichFFThandle = imshow(...
    log(R+eps),[],...
    'XData',[XYLim(1,1),XYLim(1,2)],...
    'YData',[XYLim(2,1),XYLim(2,2)]);
set(handles.fft_axes,'TickDir','out')
title('FFT Magnitude (unfiltered)');
axis image
axis on
coloring_checkbox_Callback(hObject, [], handles);
drawnow
guidata(hObject, handles);


% --------------------------------------------------------------------
% INVERSE TRANSFORM
% --------------------------------------------------------------------
function DoInvFft(hObject,eventdata,handles)
global ImgIn ImgInHue ImgInSat colorImg outImgHandle R R2 Z I2 I2C A AC

% recompose FFT & image
F2 = R2.*exp(i*Z);
I2 = real(ifft2(ifftshift(F2)));

% recompose complementary FFT & image
R2C = R-R2;
F2C = R2C.*exp(i*Z);
I2C = real(ifft2(ifftshift(F2C)));

% convert data to [0-255] scale
if max(I2(:)) - min(I2(:)) ~= 0
    if colorImg ~= 1
        I2 = uint8( (I2 - min(I2(:)))*255/(max(I2(:)) - min(I2(:))) );
    else
        % color images have [0-1] values
        I2 = uint8(I2*255);
    end
else
    % catch images with one value (ex: no filtering done yet)
    I2 = uint8(I2);
end
if max(I2C(:)) - min(I2C(:)) ~= 0
    if colorImg ~= 1
        I2C = uint8( (I2C - min(I2C(:)))*255/(max(I2C(:)) - min(I2C(:))) );
    else
        I2C = uint8(I2C*255);
    end
else
    I2C = uint8(I2C);
end

axes(handles.out_axes)
if get(handles.selection_radiobutton,'Value') == 1
    % display selection
    if get(handles.backg_checkbox,'Value') == 1 | ...
            get(handles.contrast_checkbox,'Value') == 1 | ...
            get(handles.denoise_checkbox,'Value') == 1
        % image adjust
        DoAdjust(hObject, eventdata, handles)
    else
        % no adjust
        outImgHandle = imshow(I2);
    end
else
    % display complement
    if get(handles.backg_checkbox,'Value') == 1 | ...
            get(handles.contrast_checkbox,'Value') == 1 | ...
            get(handles.denoise_checkbox,'Value') == 1
        % image adjust
        DoAdjust(hObject, eventdata, handles)
    else
        % no adjust
        outImgHandle = imshow(I2C);
    end
end
set(handles.out_axes,'TickDir','out')
title('Output image');
axis on
coloring_checkbox_Callback(hObject, [], handles);
drawnow
guidata(hObject, handles);


% --------------------------------------------------------------------
% FILTERING
% --------------------------------------------------------------------
function DoFilter(hObject,eventdata,handles)
global ImgIn XYLim R R2 Z I2 I2C M MG wichFFThandle isotop
% announce activity status
set(handles.filter_text,'BackgroundColor','r')

KeyPressed = 0;
regionNr = 0;
units = get(handles.out_axes,'units');
set(gca,'units','pixels')

% auto filtering
autopass = 0;
% auto-select +-spread of selection around max frequency
spread = str2num(get(handles.autospread_edit,'String'));

% Draw ROIs until Key pressed
while KeyPressed ~= 1
    axes(handles.fft_axes)
    
    if get(handles.auto_checkbox,'Value') == 0
        % check for key press
        % If the window is closed by the user when filtering is
        % active, this function causes Matlab to hang for a while.
        KeyPressed = waitforbuttonpress;
        
        % stop if key pressed
        if KeyPressed == 1
            break
        end
        point1 = get(gca,'CurrentPoint');    % button down detected
        finalRect = rbbox;                   % return figure units
        point2 = get(gca,'CurrentPoint');    % button up detected
        point1 = round(point1(1,1:2));       % extract x and y
        point2 = round(point2(1,1:2));
        if point1 == point2                  % discard if clicked on
            continue                         % the same place twice
        end
    else
        % auto-select
        maxFrq(1,2) = max2(isotop,1)+XYLim(2,1)-1;
        maxFrq(1,1) = max2(isotop,2)+XYLim(1,1)-1;
        point1 = maxFrq - spread;
        point2 = maxFrq + spread;
        autopass = autopass+1;
    end
    
    p1 = min(point1,point2);             % calculate locations
    offset = abs(point1 - point2);       % and dimensions
    x = [p1(1), p1(1)+offset(1), p1(1)+offset(1), p1(1), p1(1)];
    y = [p1(2), p1(2), p1(2)+offset(2), p1(2)+offset(2),  p1(2)];
    % stop if pointed outside FFT axes
    if point1(1) < XYLim(1,1) | point1(2) < XYLim(2,1) | ...
            point2(1) < XYLim(1,1) | point2(2) < XYLim(2,1) | ...
            point1(1) > XYLim(1,2) | point1(2) > XYLim(2,2) | ...
            point2(1) > XYLim(1,2) | point2(2) > XYLim(2,2)
        break
    end
    axis image
    hold on
    
    % show regions
    regionNr = regionNr+1;
    regionH(regionNr) = rectangle('Position',[x(1),y(1),offset(1),offset(2)],...
        'EdgeColor','r');
    if get(handles.elipse_radiobutton,'Value') == 1
        set(regionH(regionNr),'Curvature',[1,1]);
    else
        set(regionH(regionNr),'Curvature',[0,0]);
    end
    power = str2num(get(handles.power_edit,'String'));
    text(x(1)+offset(1),y(1)+offset(2),['\times',num2str(power)],...
        'VerticalAlignment','top','Color','r')
    
    % make filter mask - step edges
    if get(handles.elipse_radiobutton,'Value') == 1
        % elipse shape
        me = y(3)-y(1)+1;
        ne = x(2)-x(1)+1;
        [xe,ye] = meshgrid(-ne/2:(ne-1)/2,-me/2:(me-1)/2);
        fe = sqrt((xe/ne).^2 + (ye/me).^2);
        shape = fe<fe(floor(me/2)+1,1);
    else
        % rectangle shape
        shape = ones(offset(2)+1,offset(1)+1);
    end
    M(y(1)-XYLim(2,1)+1:y(3)-XYLim(2,1)+1,...
        x(1)-XYLim(1,1)+1:x(2)-XYLim(1,1)+1) = ...
        M(y(1)-XYLim(2,1)+1:y(3)-XYLim(2,1)+1,...
        x(1)-XYLim(1,1)+1:x(2)-XYLim(1,1)+1).*~shape + shape*power;
    
    % make symmetric filter
    if get(handles.symmetric_checkbox,'Value') == 1
        xs = [-x(1), -x(1)-offset(1),-x(1)-offset(1), -x(1), -x(1)];
        ys = [-y(1), -y(1),-y(1)-offset(2), -y(1)-offset(2), -y(1)];
        hold on
        
        % show regions
        regionNr = regionNr+1;
        regionH(regionNr) = ...
            rectangle('Position',[xs(2),ys(3),offset(1),offset(2)],...
            'EdgeColor','r');
        if get(handles.elipse_radiobutton,'Value') == 1
            set(regionH(regionNr),'Curvature',[1,1]);
        else
            set(regionH(regionNr),'Curvature',[0,0]);
        end
        text(xs(2)+offset(1),ys(3)+offset(2),['\times',num2str(power)],...
            'VerticalAlignment','top','Color','r')
        
        % make filter mask - step edges
        M(ys(3)-XYLim(2,1)+1:ys(3)-XYLim(2,1)+offset(2)+1,...
            xs(2)-XYLim(1,1)+1:xs(2)-XYLim(1,1)+offset(1)+1) = ...
            M(ys(3)-XYLim(2,1)+1:ys(3)-XYLim(2,1)+offset(2)+1,...
            xs(2)-XYLim(1,1)+1:xs(2)-XYLim(1,1)+offset(1)+1).*~shape + ...
            shape*power;
    end
    
    % set 'selection' radiobutton 'on' if filter is multivalued
    if numval(M) > 2 & get(handles.complement_radiobutton,'Value') == 1
        set(handles.selection_radiobutton,'Value',1);
        set(handles.complement_radiobutton,'Value',0);
    end
    
    % do inverse transform from filtered FFT
    R2 = R.*double(M);
    DoInvFft(hObject,eventdata,handles);
    
    % modify lowpass FFT if auto on
    if get(handles.auto_checkbox,'Value') == 1 & ...
            str2num(get(handles.autofrqnr_edit,'String')) >= autopass
        isotop = isotop.*double(M);
    end
    
    % show resulting FFT
    auto_togglebutton_Callback(hObject, eventdata, handles);
    
    % stop if auto-select
    if get(handles.auto_checkbox,'Value') == 1 & ...
            str2num(get(handles.autofrqnr_edit,'String')) == autopass
        KeyPressed = 1;
    end
end

% remove activity status
set(handles.filter_text,'BackgroundColor',get(gcf,'color'))
% reset FIG settings
hold off
axis image
set(handles.fft_axes,'units',units)
set(handles.out_axes,'units',units)
guidata(hObject, handles);


% --------------------------------------------------------------------
% BUTTON - FILTER
% --------------------------------------------------------------------
function Filter_pushbutton_Callback(hObject, eventdata, handles)
global ImgIn R R2 Z I2 I2C
DoFilter(hObject,eventdata,handles)
guidata(hObject, handles);


% --------------------------------------------------------------------
% BUTTON - SAVE
% --------------------------------------------------------------------
function Save_pushbutton_Callback(hObject, eventdata, handles)
global pathname filename ImgIn ImgInHue ImgInSat...
    I2 I2C A AC M MG imgNow numberOfImages colorImg
[pathstr,name,ext,versn] = fileparts(filename);

% save filtered image, complement & mask
if get(handles.backg_checkbox,'Value') == 1 | ...
        get(handles.contrast_checkbox,'Value') == 1 | ...
        get(handles.denoise_checkbox,'Value') == 1
    % adjusted image
    
    % restore colorinfo to image
    if colorImg == 1
        S = uint8(hsv2rgb(cat(3,ImgInHue,ImgInSat,double(A)/255))*255);
        SC = uint8(hsv2rgb(cat(3,ImgInHue,ImgInSat,double(AC)/255))*255);
    else
        S = A;
        SC = AC;
    end
else
    % no adjust
    
    % restore colorinfo to image
    if colorImg == 1
        % selection
        S = uint8(hsv2rgb(cat(3,ImgInHue,ImgInSat,double(I2)/255))*255);
        % complement
        SC = uint8(hsv2rgb(cat(3,ImgInHue,ImgInSat,double(I2C)/255))*255);
    else
        S = I2;
        SC = I2C;
    end
end
imwrite(S,[name,'_f.tif'])
imwrite(SC,[name,'_c.tif'])
% step mask
if max(M(:)) - min(M(:)) ~= 0
    % scale data to the [0-255] grayscale
    S = uint8(round( (M - min(M(:)))*255/(max(M(:)) - min(M(:))) ));
else
    % catch not filtered images
    S = uint8(zeros(size(M,1),size(M,2)));
end
imwrite(S,[name,'_m.tif'])

% display text on window
if get(handles.batch_checkbox,'Value') == 0
    saveText = {['The filtered image, its complement'],...
            ['and the filtering mask were saved to:'],...
            [''],...
            [pathname],...
            ['   ',name,'_f.tif'],...
            ['   ',name,'_c.tif'],...
            ['   ',name,'_m.tif']};
    warndlg(saveText,'Save filtered images');
else
    if imgNow == numberOfImages
        saveText = {['The filtered images, their complements'],...
                ['and the filtering masks were saved to:'],...
                [''],...
                [pathname],...
                ['   [image name]_f.tif'],...
                ['   [image name]_c.tif'],...
                ['   [image name]_m.tif']};
        warndlg(saveText,'Save filtered images');
    end
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% BUTTON - UNDO
% --------------------------------------------------------------------
function Undo_pushbutton_Callback(hObject, eventdata, handles)
global ImgIn R R2 M MG
% announce activity status
set(handles.filter_text,'BackgroundColor','r')

% set toolbox preferences
iptsetpref('ImshowAxesVisible','on')

% display FFT & inverse transform
axes(handles.fft_axes), cla
axes(handles.out_axes), cla
DoFft(hObject,handles);

% reset mask to neutral
M = ones(size(ImgIn,1),size(ImgIn,2));
R2 = R;
DoInvFft(hObject,eventdata,handles);
DoFilter(hObject,eventdata,handles);

% reset auto lowpass filter
if get(handles.auto_checkbox,'Value') == 1
    auto_checkbox_Callback(hObject, eventdata, handles);
end

% reset toolbox preferences
iptsetpref('ImshowAxesVisible','off')
guidata(hObject, handles);


% --------------------------------------------------------------------
% BUTTONS - HELP
% --------------------------------------------------------------------
function help_pushbutton_Callback(hObject, eventdata, handles)
web(['file:' which('bluenile/help.html')],'-browser');


% --------------------------------------------------------------------
% FILTER SHAPE - DISK
% --------------------------------------------------------------------
function elipse_radiobutton_Callback(hObject, eventdata, handles)

if get(handles.elipse_radiobutton,'Value') == 1
    set(handles.rectangle_radiobutton,'Value',0);
else
    set(handles.elipse_radiobutton,'Value',1);
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% FILTER SHAPE - RECTANGLE
% --------------------------------------------------------------------
function rectangle_radiobutton_Callback(hObject, eventdata, handles)
if get(handles.rectangle_radiobutton,'Value') == 1
    set(handles.elipse_radiobutton,'Value',0);
else
    set(handles.rectangle_radiobutton,'Value',1);
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% FILTER SHAPE - SYMMETRIC
% --------------------------------------------------------------------
function symmetric_checkbox_Callback(hObject, eventdata, handles)


% --------------------------------------------------------------------
% FILTER SHAPE - POWER
% --------------------------------------------------------------------
function power_edit_Callback(hObject, eventdata, handles)


% --------------------------------------------------------------------
% AUTO FILTER - LOWPASS FILTER
% --------------------------------------------------------------------
function auto_checkbox_Callback(hObject, eventdata, handles)
global ImgIn isotop0 isotop
if get(handles.auto_checkbox,'Value') == 1
    set(handles.batch_checkbox,'Enable','on');
    
    % pseudo-isotopic low-pass filtering
    % (pops up mid-frequencies by superposing the FFT's
    % of the NS & the EW embossed image)
    Rns = abs(fftshift(fft2(emboss(ImgIn))));
    Rew = abs(fftshift(fft2(imrotate(emboss(imrotate(ImgIn,90)),-90))));
    isotop = Rns + Rew;
    isotop(max2(isotop,1),max2(isotop,2)) = 0;
    isotop0 = isotop;
else
    set(handles.batch_checkbox,'Enable','off');
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% AUTO FILTER - BATCH
% --------------------------------------------------------------------
function batch_checkbox_Callback(hObject, eventdata, handles)


% --------------------------------------------------------------------
% AUTO FILTER - PASSES
% --------------------------------------------------------------------
function autofrqnr_edit_Callback(hObject, eventdata, handles)

innr = str2num(get(handles.autofrqnr_edit,'String'));
% Passes must be positive integers
if innr < 1 | innr-abs(innr) ~= 0
    warndlg({'Values must be positive integers'},...
        'Warning');
    return
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% AUTO FILTER - SPREAD
% --------------------------------------------------------------------
function autospread_edit_Callback(hObject, eventdata, handles)

innr = str2num(get(handles.autospread_edit,'String'));
% Passes must be positive integers
if innr < 1 | innr-abs(innr) ~= 0
    warndlg({'Values must be positive integers'},...
        'Warning');
    return
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% REGION - SELECTION
% --------------------------------------------------------------------
function selection_radiobutton_Callback(hObject, eventdata, handles)
global I2 A M MG outImgHandle
if get(handles.selection_radiobutton,'Value') == 1
    set(handles.complement_radiobutton,'Value',0);
    
    axes(handles.out_axes)
    % display selection
    if get(handles.backg_checkbox,'Value') == 1 | ...
            get(handles.contrast_checkbox,'Value') == 1 | ...
            get(handles.denoise_checkbox,'Value') == 1
        % image adjust
        outImgHandle = imshow(A);
    else
        % no adjust
        outImgHandle = imshow(I2);
    end
    axis on
    title('Output image')
    coloring_checkbox_Callback(hObject, [], handles);
    drawnow
else
    set(handles.selection_radiobutton,'Value',1);
    
    % don't toogle radiobuttons if filter is multivalued
    if numval(M) > 2, return, end

    complement_radiobutton_Callback(hObject, eventdata, handles)
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% REGION - COMPLEMENT
% --------------------------------------------------------------------
function complement_radiobutton_Callback(hObject, eventdata, handles)
global I2C AC M MG outImgHandle

% don't show if filter is multivalued
if numval(M) > 2
    set(handles.complement_radiobutton,'Value',0);
    msgbox('Multivalued filters don''t have complements.',...
        'Warning','error')
    return
end
    
if get(handles.complement_radiobutton,'Value') == 1
    set(handles.selection_radiobutton,'Value',0);
    
    axes(handles.out_axes)
    cla
    % display complement
    if get(handles.backg_checkbox,'Value') == 1 | ...
            get(handles.contrast_checkbox,'Value') == 1 | ...
            get(handles.denoise_checkbox,'Value') == 1
        % image adjust
        outImgHandle = imshow(AC);
    else
        % no adjust
        outImgHandle = imshow(I2C);
    end
    axis on
    title('Output image')
    coloring_checkbox_Callback(hObject, [], handles);
    drawnow
else
    set(handles.complement_radiobutton,'Value',1);
    selection_radiobutton_Callback(hObject, eventdata, handles)
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% VARIA - EVEN BACKGROUND
% --------------------------------------------------------------------
function backg_checkbox_Callback(hObject, eventdata, handles)
DoAdjust(hObject, eventdata, handles)
guidata(hObject, handles);


% --------------------------------------------------------------------
% VARIA - ADJUST HISTOGRAM
% --------------------------------------------------------------------
function contrast_checkbox_Callback(hObject, eventdata, handles)
DoAdjust(hObject, eventdata, handles)
guidata(hObject, handles);


% --------------------------------------------------------------------
% VARIA - DENOISE
% --------------------------------------------------------------------
function denoise_checkbox_Callback(hObject, eventdata, handles)
DoAdjust(hObject, eventdata, handles)
guidata(hObject, handles);


% --------------------------------------------------------------------
% VARIA - ADJUST FUNCTION
% --------------------------------------------------------------------
function DoAdjust(hObject, eventdata, handles)
global I2 I2C A AC colorImg ImgInHue ImgInSat outImgHandle
% anounce start, cause process can be looong
set(handles.adjust_text,'BackgroundColor','r')
drawnow

axes(handles.out_axes)
A = I2;
AC = I2C;

if get(handles.backg_checkbox,'Value') == 1
    % Background leveling
    SE = strel('disk',7);
    A = imsubtract(imadd(A,imtophat(A,SE)),imbothat(A,SE));
    AC = imsubtract(imadd(AC,imtophat(AC,SE)),imbothat(AC,SE));
end
if get(handles.contrast_checkbox,'Value') == 1
    % Contrast adjustment
    A = histeq(A);
    AC = histeq(AC);
end
if get(handles.denoise_checkbox,'Value') == 1
    % Denoising
    A = medfilt2(wiener2(A));
    if numval(AC)~=1
        AC = medfilt2(wiener2(AC));
    end
end

% display result
if get(handles.selection_radiobutton,'Value') == 1
    outImgHandle = imshow(A);
else
    outImgHandle = imshow(AC);
end

% set colormap
coloring_checkbox_Callback(hObject, [], handles);

% rest of display settings
set(handles.out_axes,'TickDir','out')
title('Output image');
axis on

% anounce end
set(handles.adjust_text,'BackgroundColor',get(gcf,'Color'))
drawnow

guidata(hObject, handles);


% --------------------------------------------------------------------
% VARIA - COLORING
% --------------------------------------------------------------------
function coloring_checkbox_Callback(hObject, eventdata, handles)
global colorImg inImgHandle outImgHandle ImgInHue ImgInSat

if get(handles.coloring_checkbox,'Value') == 1
    if colorImg == 1
        % display pseudo-colors of grayscale of color image
        % input image
        imgTemp = rgb2hsv(get(inImgHandle,'CData'));
        if length(size(imgTemp)) == 3
            imgTemp = imgTemp(:,:,3);
            imgTemp = uint8( (imgTemp - ...
                min(imgTemp(:)))*255/(max(imgTemp(:)) - min(imgTemp(:))) );
            set(inImgHandle,'CData',imgTemp);
        end
        
        % output image
        if ishandle(outImgHandle) == 1
            imgTemp = rgb2hsv(get(outImgHandle,'CData'));
            if length(size(imgTemp)) == 3
                imgTemp = imgTemp(:,:,3);
                imgTempRange = max(imgTemp(:)) - min(imgTemp(:));
                % catch not filtered images
                if imgTempRange == 0
                    imgTempRange = eps;
                end
                imgTemp = uint8( (imgTemp - min(imgTemp(:)))*255/imgTempRange );
                set(outImgHandle,'CData',imgTemp);
            end
        end
    end
    colormap(jet)
else
    if colorImg == 1
        % restore color content of color image
        imgTemp = get(inImgHandle,'CData');
        if length(size(imgTemp)) == 2
            imgTemp = hsv2rgb(cat(3,ImgInHue,ImgInSat,double(imgTemp)/255));
            set(inImgHandle,'CData',imgTemp);
        end
        
        if ishandle(outImgHandle) == 1
            imgTemp = get(outImgHandle,'CData');
            imgTemp = hsv2rgb(cat(3,ImgInHue,ImgInSat,double(imgTemp)/255));
            set(outImgHandle,'CData',imgTemp);
        end
    end
    colormap(gray)
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% VARIA - PIXEL COORDS & VALUES
% --------------------------------------------------------------------
function Pixval_pushbutton_Callback(hObject, eventdata, handles)
units = get(gcf,'units');
set(gcf,'units','pixels')
pixval
set(gcf,'units',units)


% --------------------------------------------------------------------
% VARIA - IMAGE PROFILE
% --------------------------------------------------------------------
function Profile_pushbutton_Callback(hObject, eventdata, handles)
units = get(gcf,'units');
set(gcf,'units','pixels')
improfile
set(gcf,'units',units)


% --------------------------------------------------------------------
% VARIA - ZOOM
% --------------------------------------------------------------------
function Zoom_togglebutton_Callback(hObject, eventdata, handles)

axes(handles.fft_axes);
if get(handles.Zoom_togglebutton,'Value') == 1
    zoom on
else
    zoom off
end


% --------------------------------------------------------------------
% VARIA - SHOW FILTER
% --------------------------------------------------------------------
function ShowFilter_pushbutton_Callback(hObject, eventdata, handles)
global M MG
figure('Name','Displaying filter',...
    'NumberTitle','off',...
    'Menubar','none');
if get(handles.selection_radiobutton,'Value') == 1
    mesh(M)
else
    mesh(double(~M))
end
axis tight
view(-38,30)
rotate3d on


% --------------------------------------------------------------------
% VARIA - CHOOSE WHICH FFT TO DISPLAY
% --------------------------------------------------------------------
function WhichFFT_togglebutton_Callback(hObject, eventdata, handles)
global R R2 wichFFThandle

axes(handles.fft_axes);
if get(handles.WhichFFT_togglebutton,'Value') == 0
    set(wichFFThandle,'CData',log(R));
    title('FFT Magnitude (unfiltered)');
else
    set(wichFFThandle,'CData',log(R2+eps));
    title('FFT Magnitude (filtered)');
end
guidata(hObject, handles);


% --------------------------------------------------------------------
% VARIA - SHOW LOWPASS FILTER
% --------------------------------------------------------------------
function auto_togglebutton_Callback(hObject, eventdata, handles)
global isotop0 wichFFThandle R R2

axes(handles.fft_axes);
if get(handles.auto_togglebutton,'Value') == 0
    % display not-lowpass filter FFT
    if get(handles.WhichFFT_togglebutton,'Value') == 0
        set(wichFFThandle,'CData',log(R));
        title('FFT Magnitude (unfiltered)');
    else
        set(wichFFThandle,'CData',log(R2+eps));
        title('FFT Magnitude (filtered)');
    end
else
    % display FFT of lowpass filter
    if get(handles.auto_checkbox,'Value') == 1
        % only if 'auto' is on
        set(wichFFThandle,'CData',log(isotop0+eps));
        title('FFT Magnitude (lowpass filtered)');
    else
        % else do nothing
        set(handles.auto_togglebutton,'Value',0)
    end
end
guidata(hObject, handles);




% % ====================================================================
% % DISABLED FUNCTIONALITY : TAPING EDGES
% 
% % --------------------------------------------------------------------
% % TAP FILTER EDGES
% % --------------------------------------------------------------------
% function DoTap(hObject, handles)
% global ImgIn M MG
% 
% % anounce start, cause process can be looong
% set(handles.taper_checkbox,'BackgroundColor','r')
% drawnow
% 
% % tap
% % figure's edges are not tapered
% PSF = fspecial('gaussian',...
%     str2num(get(handles.gsize_edit,'String')),...
%     str2num(get(handles.power_edit,'String')));
% fig_edges = imfilter(ones(size(ImgIn,1),size(ImgIn,2)),PSF);
% roi_edges = imfilter(M,PSF); % < OLD
% 
% MG = 1-fig_edges+roi_edges;             % < no noticeable change from not taped
% 
% % anounce end
% set(handles.taper_checkbox,'BackgroundColor',...
%     get(gcf,'Color'))
% drawnow
% guidata(hObject, handles);
% 
% 
% % --------------------------------------------------------------------
% % FILTER SHAPE - TAPER FUNCTION
% % --------------------------------------------------------------------
% function taper_checkbox_Callback(hObject, eventdata, handles)
% global R R2 M MG
% 
% % anounce start, cause process can be looong
% set(handles.taper_checkbox,'BackgroundColor','r')
% drawnow
% 
% axes(handles.out_axes)
% if get(handles.taper_checkbox,'Value') == 1
%     % tap filter edges
%     DoTap(hObject,handles);
%     % filter FFT
%     R2 = R.*MG;
% else
%     % don't tap filter edges
%     R2 = R.*M;
% end
% 
% set(handles.taper_checkbox,'BackgroundColor','r')
% drawnow
% 
% % display results
% DoInvFft(hObject,eventdata,handles);
% 
% % anounce end
% set(handles.taper_checkbox,'BackgroundColor',...
%     get(gcf,'Color'))
% drawnow
% 
% guidata(hObject, handles);
% 
% 
% % --------------------------------------------------------------------
% % FILTER SHAPE - TAPER PARAMETERS
% % --------------------------------------------------------------------
% % --- Executes on edit in gsize_edit.
% function gsize_edit_Callback(hObject, eventdata, handles)
% global ImgIn
% 
% gsize = str2num(get(handles.gsize_edit,'String'));
% % catch values bigger than half the image
% if gsize > floor(min(size(ImgIn,1),size(ImgIn,2))/2)
%     warndlg({['Values must be smaller than'],...
%             ['half the smallest image side.']},...
%         'Warning');
% end
% guidata(hObject, handles);
% 
% 


