function [num,val] = numval(IN,datatype)
% NUMVAL counts and lists the different values in a matrix
% IN		- input matrix
% datatype	- signifies all values are real and positive
% 			  (datatype can have any value, say '1' for simplicity)
%			  The algorithm is faster if you specify this.
% num		- number of different values
% val       - list of all different values (sorted low to high)
%
% --------------------------------------------- 
% ATTENTION:
% Matlab 7.4 has a function which does the same and is much faster:
% b = unique(A);          - lists the values
% b = length(unique(A));  - get the number of different values
% --------------------------------------------- 
%
% Vlad Atanasiu - 09/09/2003
% Revisions: 2  > 01/07/2007

num = 0;
val = [];
IN = IN(:);
if nargin == 1  % unspecified values type
	while isempty(IN) ~= 1
        if nargout == 2
            val = [val IN(1)];
        end
		IN(find(IN==IN(1))) = [];
		num = num + 1;
    end
    val = sort(val);
else            % values are real and positive
	num = length(find(sum(histc(IN,0:max(max(IN))),2)));
    if nargout == 2
        for k = min(IN):max(IN)
            if isempty(find(IN == k, 1)) ~= 1;
                val = [val k];
            end
        end
    end
end

